@@ -9,7 +9,7 @@ This program comes with ABSOLUTELY NO WARRANTY; for details type `show w'.
 This is free software, and you are welcome to redistribute it
 under certain conditions; type `show c' for details.
 
- This program is free software: you can redistribute it and/or modify
+This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
@@ -27,8 +27,9 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <QDesktopServices>
 #include <QDebug>
 #include <QCoreApplication>
+#include <QGuiApplication>
 #include <QSettings>
-
+#include <QProcess>
 #include <MauiKit4/Core/fmh.h>
 
 #include <MauiKit4/FileBrowsing/tagging.h>
@@ -39,7 +40,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 Q_GLOBAL_STATIC(Pix, pixInstance)
 
 Pix::Pix(QObject *parent) : QObject(parent)
-  , m_allImagesModel(nullptr)
+    , m_allImagesModel(nullptr)
 {
 }
 
@@ -80,9 +81,9 @@ Gallery *Pix::allImagesModel()
         m_allImagesModel->setRecursive(true);
         m_allImagesModel->componentComplete(); //call this to actually get the data
         connect(this, &Pix::sourcesChanged, [this]()
-        {
-            m_allImagesModel->setUrls(QUrl::fromStringList(sources()));
-        });
+                {
+                    m_allImagesModel->setUrls(QUrl::fromStringList(sources()));
+                });
     }
 
     qDebug() << "REQUEST ALL IMAGES MODEL & EXISTING ONE";
@@ -92,7 +93,7 @@ Gallery *Pix::allImagesModel()
 
 Pix *Pix::instance()
 {
-   return pixInstance();
+    return pixInstance();
 }
 
 const QStringList Pix::getSourcePaths()
@@ -156,6 +157,17 @@ void Pix::showInFolder(const QStringList &urls)
         QDesktopServices::openUrl(FMStatic::fileDir(url));
 }
 
+void Pix::createNote(const QString &text)
+{
+    QString program = "buho";
+    QStringList arguments;
+    arguments << "-n" << "-c" << text.toUtf8();
+
+    QProcess *myProcess = new QProcess();
+    myProcess->startDetached(program, arguments);
+    // myProcess->deleteLater();
+}
+
 void Pix::addSources(const QStringList &paths)
 {
     saveSourcePath(paths);
@@ -167,3 +179,43 @@ void Pix::removeSources(const QString &path)
     removeSourcePath(path);
     Q_EMIT sourcesChanged();
 }
+
+FileWatcher::FileWatcher(QObject *parent) : QObject(parent)
+    ,m_watcher(new QFileSystemWatcher(this))
+{
+    connect(m_watcher, &QFileSystemWatcher::fileChanged, this, &FileWatcher::onFileChanged);
+}
+
+QString FileWatcher::url() const
+{
+    return m_url;
+}
+
+void FileWatcher::setUrl(const QString &newUrl)
+{
+    if (m_url == newUrl)
+        return;
+    m_url = newUrl;
+    qDebug()  << "new file to watch" << m_url;
+    m_watcher->removePaths(m_watcher->files());
+    m_watcher->addPath(QUrl(m_url).toLocalFile());
+
+    Q_EMIT urlChanged();
+}
+
+void FileWatcher::onFileChanged(const QString &url)
+{
+    if(FMH::fileExists(QUrl::fromLocalFile(url)))
+    {
+        qDebug() << "The watched file was removed" << url;
+
+        Q_EMIT fileModified();
+    }else
+    {
+        qDebug() << "The watched file was changed" << url;
+
+        m_watcher->removePaths(m_watcher->files());
+        setUrl({});
+        Q_EMIT fileDeleted();
+    }
+}
